@@ -38,14 +38,15 @@ class GP(Heuristics):
         evaluation_function (Callable[[Any], float]): The fitness function. Should take in any valid individual and return a float. The default is that the higher the fitness, the more applicable is the solution to the problem. Turn on the parameter minimize to switch it around.
         minimize (bool): When switch on, the fitness function is reversed, so that a higher result from the fitness function corresponds to a less fit solution (default = False).
         target_fitness (Optional[float]): Sets a target fitness. When this fitness is reached, the algorithm stops running (default = None).
-        favor_less_deep_trees (bool): If set to True, this gives a tiny penalty to deeper trees to favor simpler trees (default = False).
+        favor_less_complex_trees (bool): If set to True, this gives a tiny penalty to more complex (with more nodes) trees to favor simpler trees (default = False).
         randomSource (Callable[[int], RandomSource]): The random source function used by the program. Should take in an integer, representing the seed, and return a RandomSource.
         population_size (int): The population size (default = 200). Apart from the first generation, each generation the population is made up of the elites, novelties, and transformed individuals from the previous generation. Note that population_size > (n_elites + n_novelties + 1) must hold.
         n_elites (int): Number of elites, i.e. the number of best individuals that are preserved every generation (default = 5).
         n_novelties (int): Number of novelties, i.e. the number of newly generated individuals added to the population each generation. (default = 10).
         number_of_generations (int): Number of generations (default = 100).
         max_depth (int): The maximum depth a tree can have (default = 15).
-        max_init_depth (int): The maximum depth a tree can have in the initialisation population. Currently only working for treebased representation (default = max_depth).
+        max_init_depth (int): The maximum depth a tree can have in the initialisation population. Currently only working for tree-based representation and dynamic SGE (default = max_depth).
+        min_init_depth (int): The minimum depth a tree can have in the initialisation population. Only relevant when using the Random_Production initialization method (default = None).
         selection_method (Tuple[str, int]): Allows the user to define the method to choose individuals for the next population (default = ("tournament", 5)).
         ramped_half_and_half (bool): Specify whether you want the population to be initialized ramped half and half. Currently only working for treebased representation (default = True).
         
@@ -103,15 +104,15 @@ class GP(Heuristics):
         grammar: Grammar,
         representation: Representation = treebased_representation,
         # These classes should be replaced by the problem alone.
-        problem: Problem = None,
+        problem: Problem | None = None,
         evaluation_function: Callable[
             [Any],
             float,
-        ] = None,  # DEPRECATE in the next version
+        ] | None = None,  # DEPRECATE in the next version
         minimize: bool = False,  # DEPRECATE in the next version
         target_fitness: float | None = None,  # DEPRECATE in the next version
         evolve_grammar: EvolveGrammar | None = None,
-        favor_less_deep_trees: bool = False,  # DEPRECATE in the next version
+        favor_less_complex_trees: bool = True,  # DEPRECATE in the next version
         randomSource: Callable[[int], RandomSource] = RandomSource,
         population_size: int = 200,
         n_elites: int = 5,
@@ -119,6 +120,7 @@ class GP(Heuristics):
         number_of_generations: int = 100,
         max_depth: int = 15,
         max_init_depth: int | None = None,
+        min_init_depth: int | None = None,
         selection_method: tuple[str, int] = ("tournament", 5),
         ramped_half_and_half: bool = False,
         # -----
@@ -127,8 +129,8 @@ class GP(Heuristics):
         probability_crossover: float = 0.9,
         either_mut_or_cro: float | None = None,
         hill_climbing: bool = False,
-        specific_type_mutation: type = None,
-        specific_type_crossover: type = None,
+        specific_type_mutation: type | None = None,
+        specific_type_crossover: type | None = None,
         depth_aware_mut: bool = False,
         depth_aware_co: bool = False,
         # -----
@@ -138,21 +140,21 @@ class GP(Heuristics):
         timer_stop_criteria: bool = False,
         timer_limit: int = 60,
         # -----
-        save_to_csv: str = None,
+        save_to_csv: str | None = None,
         save_genotype_as_string: bool = True,
         test_data: Callable[
             [Any],
             float,
-        ] = None,
+        ] | None = None,
         only_record_best_inds: bool = True,
         # -----
-        callbacks: list[Callback] = None,
+        callbacks: list[Callback] | None = None,
     ):
         assert population_size > (n_elites + n_novelties + 1)
 
         self.problem: Problem = wrap_depth(
             process_problem(problem, evaluation_function, minimize, target_fitness),
-            favor_less_deep_trees,
+            favor_less_complex_trees,
         )
 
         self.grammar = grammar
@@ -167,8 +169,11 @@ class GP(Heuristics):
             self.max_init_depth = max_init_depth
         else:
             self.max_init_depth = self.max_depth
+        if min_init_depth:
+            assert min_init_depth <= self.max_init_depth
+        self.min_init_depth = min_init_depth
         self.evolve_grammar = evolve_grammar
-        self.favor_less_deep_trees = favor_less_deep_trees
+        self.favor_less_complex_trees = favor_less_complex_trees
         self.novelty = selection.create_novelties(
             self.create_individual,
             max_depth=max_depth,
@@ -235,7 +240,7 @@ class GP(Heuristics):
                         grammar,
                         individual.genotype,
                     )
-                    test_fitness = test_data(phenotype)  # type: ignore
+                    test_fitness = test_data(phenotype) # type: ignore
                     return test_fitness
 
                 self.test_data = test_evaluate
@@ -350,10 +355,7 @@ class GP(Heuristics):
                     inds = npop
                 best = self.get_best_individual(self.problem, inds)
                 prob = best.production_probabilities(
-                    lambda x: self.representation.genotype_to_phenotype(
-                        self.grammar,
-                        x,
-                    ),
+                    self.representation.genotype_to_phenotype,
                     self.grammar,
                 )
                 self.grammar = self.grammar.update_weights(
@@ -376,6 +378,8 @@ class GP(Heuristics):
         )
 
     def init_population(self, ramped_half_and_half):
+        self.representation.method.min_depth = self.min_init_depth
+        
         if ramped_half_and_half:
             n_not_ramped = int(self.population_size / 2)
             n_ramped = self.population_size - n_not_ramped 
@@ -388,9 +392,14 @@ class GP(Heuristics):
                 for _ in range(n_not_ramped)
             ]
                 
-            return pop_ramped + pop_not_ramped
+            pop = pop_ramped + pop_not_ramped
         else:
-            return [
+            pop = [
                 self.create_individual(self.max_init_depth)
                 for _ in range(self.population_size)
             ]
+        
+        self.representation.depth = self.max_depth
+        self.representation.method.min_depth = None
+        return pop
+        
